module Simulation

use GlobalVariables
use ErrorHandling

implicit none

    contains

! RunSimulation -> Used in MAIN
!
! NeoPopGrowth  -> Used by RunSimulation
! NeoPopDispersion  -> Used by RunSimulation
! SaveOutputFile  (currently not used -> Previously used to debug)
! SaveFrontPositionRegion(x0, y0, xf, yf) -> Used by RunSimulation
! SaveSites(iIterations) -> Used by RunSimulation

subroutine RunSimulation(maxIterations, reprRateNeo)
    ! Calls the Neolithic population growth and population dispersal routines at every iteration
    !
    ! Used by Main
    !
    ! Uses NeoPopGrowth, NeoPopDispersion
    ! Uses SaveOutputFile, SaveFrontPosition, SaveSites
    
    implicit none
    integer(4), intent(in) :: maxIterations
    real(8), intent(in) :: reprRateNeo
    integer(4) :: iIterations
    
        do iIterations = 1, maxIterations
                        
            call NeoPopDispersion(iIterations)         
            call NeoPopGrowth(reprRateNeo, iIterations)
            call SaveOutputFile(iIterations)
            call SaveSites(iIterations)

            print *, 'iteration', iIterations
            
        enddo

    call SaveFrontPositionRegion(1, 1, 180, 102)
    
end subroutine RunSimulation


subroutine NeoPopGrowth(reprRateNeo, iIterations)
! Compute the hunter-gatherer, farmer N and farmer X population density after 
! vertical cultural transmission and population growth process.
! 
! Sets values higher than the carrying capacity (maxNeoPop) at maxNeoPop
! Records front arrival when the farmer population is higher than the threshold
!
! Used by RunSimulation

    implicit none
    real(8), intent(in) :: reprRateNeo
    integer(4), intent(in) :: iIterations
    integer :: iNode, j
    real(8) :: localFPop, interchange, couplesHF, couplesHN, couplesHX
    real(8) :: FPalRepr, FNeoRepr, couplesNX, couplesNN, couplesXX
    
    do j = 1, numHabitableNodes  !only habitable nodes can be inhabited
        iNode = habitableNodes(j)
        localFPop = Nodes(iNode)%FPalPop + Nodes(iNode)%FNeoPop
        if (localFPop > 0) then 
            !vertical cultural transmission: 
            !couplesHF = couplesHN + couplesHX = interbreeding * HGPop * localFPop / (HGPop + localFPop)
            interchange = Nodes(iNode)%HGPop + localFPop
            
            couplesHF = interbreeding*localFPop*Nodes(iNode)%HGPop/interchange
            couplesHX = interbreeding*Nodes(iNode)%FPalPop*Nodes(iNode)%HGPop/interchange
            couplesHN = interbreeding*Nodes(iNode)%FNeoPop*Nodes(iNode)%HGPop/interchange
                        
            !HG population after interaction with farmers
            Nodes(iNode)%HGPop = (Nodes(iNode)%HGPop - couplesHF)*reprRateHG
            
            !Farmers remaining after the cultural process
            FPalRepr = Nodes(iNode)%FPalPop - couplesHX
            FNeoRepr = Nodes(iNode)%FNeoPop - couplesHN

            !Mixed couples among farmers Neo and Meso
            !couplesNX = N * X / (N+X)
            couplesNX = (FPalRepr * FNeoRepr) / (FPalRepr + FNeoRepr)
            
            !Remaining couples of the same kind
            couplesNN = (FNeoRepr - couplesNX) / 2.
            couplesXX = (FPalRepr - couplesNX) / 2.
            
            !Final farmer population 
            Nodes(iNode)%FPalPop = reprRateNeo*(2.0*couplesXX + couplesNX + 2.0*couplesHX + couplesHN)
            Nodes(iNode)%FNeoPop = reprRateNeo*(2.0*couplesNN + couplesNX + couplesHN)
        end if
    enddo

    !maximum control population and record of front arrival
    do j = 1, numHabitableNodes
        iNode = habitableNodes(j)
        localFPop = Nodes(iNode)%FPalPop + Nodes(iNode)%FNeoPop
 
        if (localFPop > 0) then
            if (localFPop >= maxFPop) then
                Nodes(iNode)%FPalPop = Nodes(iNode)%FPalPop * maxFPop / localFPop
                Nodes(iNode)%FNeoPop = Nodes(iNode)%FNeoPop * maxFPop / localFPop
            end if
            if (localFPop > threshold) then
                if (Nodes(iNode)%frontArrival == 500) Nodes(iNode)%frontArrival = iIterations
            endif
        end if
        
        if (Nodes(iNode)%HGPop > 0) then
            if (Nodes(iNode)%HGPop >= maxHGPop) then
                Nodes(iNode)%HGPop = maxHGPop
            endif
        endif

    end do
end subroutine NeoPopGrowth

subroutine NeoPopDispersion(iIterations)
    ! Compute the farmer populations N and X after dispersion
    ! 
    ! Sets values higher than the carrying capacity (maxNeoPop) at maxNeoPop
    ! Records front arrival when the farmer population is higher than the threshold
    !
    ! Used by RunSimulation
    implicit none
    integer(4), intent(in) :: iIterations !iteration number
    integer(4) :: i, j, iNode
    integer(4) :: numDest, destination
    real(8) :: destPop, destNeoPop, destPalPop
    real(8) :: farmerPop, initialFNeo, initialFPal
    real(8) :: dens, densNeo, densPal
    
    !Dispersion of farmers (N and X genetics)
    do i = 1, numHabitableNodes
        iNode = habitableNodes(i)
        farmerPop = Nodes(iNode)%FNeoPop + Nodes(iNode)%FPalPop
        initialFNeo = Nodes(iNode)%FNeoPop
        initialFPal = Nodes(iNode)%FPalPop
        
        if (farmerPop > 0) then
            densNeo = initialFNeo !total farmer N population at the initial node
            densPal = initialFPal !total farmer X population at the initial node
            
            !Dispersion destinations
            numDest = Nodes(iNode)%numDestinations
            
            do j = 1, numDest
                destination = Nodes(iNode)%destinations(j)
                
                !Population that moves to destination
                destNeoPop = densNeo*(Nodes(iNode)%destinationWeight(j)) 
                destPalPop = densPal*(Nodes(iNode)%destinationWeight(j))
                
                !New total population at the destination after migration
                Nodes(destination)%FNeoPop2 = Nodes(destination)%FNeoPop2 + destNeoPop 
                Nodes(destination)%FPalPop2 = Nodes(destination)%FPalPop2 + destPalPop 

                !Population that has not moved yet
                initialFNeo = initialFNeo - destNeoPop 
                initialFPal = initialFPal - destPalPop
                
            enddo    
            
            !Population that remains at the initial node
            Nodes(iNode)%FNeoPop2 = Nodes(iNode)%FNeoPop2 + initialFNeo
            Nodes(iNode)%FPalPop2 = Nodes(iNode)%FPalPop2 + initialFPal
                        
        endif
    enddo

    
    ! Set final population after dispersion.
    ! Record arrival if farmer population > threshold
    do i = 1, numHabitableNodes
        iNode = habitableNodes(i)
        
        ! Final population after dispersion
        Nodes(iNode)%FNeoPop = Nodes(iNode)%FNeoPop2
        Nodes(iNode)%FPalPop = Nodes(iNode)%FPalPop2
        
        ! Record arrival
        if (Nodes(iNode)%FNeoPop + Nodes(iNode)%FPalPop > threshold) then
            if (Nodes(iNode)%frontArrival == 500) Nodes(iNode)%frontArrival = iIterations
        endif
        
        ! Set computing variable to 0
        Nodes(iNode)%FNeoPop2 = 0.
        Nodes(iNode)%FPalPop2 = 0.
    enddo 
    
end subroutine NeoPopDispersion

subroutine SaveOutputFile(iIteracions)
    ! Records each subpopulation and the fraction of haplogroup K at each node and interation
    ! Saves a file per iteration
    !
    !Used by RunSimulation

    implicit none
    integer :: iIteracions
    integer :: iNode
    real(8) :: percentatge
    character(len=3) :: str
    character(len=20) :: name
    
    !Name the file as the current iteration
    write (str, '(i3)') iIteracions
    name = 'results'//str//'gen.txt'

    !Save data
    open (13, file=name, status="unknown")
    write (13,fmt="(a5, 2a15, 3a8, a15)") 'Node','Longitude', 'Latitude','HG_Pop', 'X_Pop','N_Pop', 'percentage'
    
    do iNode = 1, numNodes
        !%K at the node
        percentatge = 100. * Nodes(iNode)%FNeoPop/(Nodes(iNode)%FPalPop + Nodes(iNode)%FNeoPop)
        ! Save results
        write (13, fmt="(i5,2f15.8, 3i8, f15.8)") Nodes(iNode)%node_id, Nodes(iNode)%longitude, Nodes(iNode)%latitude, & 
    & nint(Nodes(iNode)%HGPop), nint(Nodes(iNode)%FPalPop), nint(Nodes(iNode)%FNeoPop), percentatge
    enddo

    close(13)
end subroutine SaveOutputFile

subroutine SaveFrontPositionRegion(x0, y0, xf, yf)
    ! Save the arrival time at each node within a certain region of the grid
    !
    !Used by RunSimulations
    implicit none
    integer(4), intent(in) :: x0, y0, xf, yf
    integer(4) :: i, j, iNode
        
    open (14, file = "front.dat", status = "unknown")

    write (14, fmt="(a5, 2a15, 3a8)") "Node", "Longitude", "Latitude", "X(km)", "Y(km)", "Arriv"    
    do j = y0, yf
        do i = x0, xf
            iNode = nodesGrid(i,j)
            if (Nodes(iNode)%node_type /= 0) then   !!Save locations not on the sea
                !write (14, fmt="(i5,2f15.8,i5,2f15.8)") Nodes(iNode)%node_id, Nodes(iNode)%longitude, Nodes(iNode)%latitude, Nodes(iNode)%frontArrival&
                !&, Nodes(iNode)%FPalPop, Nodes(iNode)%FNeoPop
                write (14, fmt="(i5,2f15.8,3i8)") Nodes(iNode)%node_id, Nodes(iNode)%longitude, Nodes(iNode)%latitude, 50*Nodes(iNode)%x, &
                    & 50*Nodes(iNode)%y, Nodes(iNode)%frontArrival
            endif
        enddo    
    enddo
    close (14)
endsubroutine

subroutine SaveSites(iIterations)
    ! Save the population and %K at the locations and date of the genetic data
    ! considering that the simulation starts at 8233 BCE
    !
    ! Used by RunSimulation
    integer, intent(in) :: iIterations
    integer :: node, file_err
    real(8) :: fraction

    select case (iIterations)
    case (30)
        open (55, file="Kfraction.txt", status="unknown", iostat=file_err)
        call OpenFileError(file_err, "There has been an error while opening 'Kfraction.txt' file.")
        write(55,fmt="(a8,a15, 4a8)") "Node", "Region", "HG pop", "X pop", "N pop", "Kfrac"
        node = 5695
        fraction = 100. * Nodes(node)%FNeoPop / (Nodes(node)%FPalPop + Nodes(node)%FNeoPop)
        write(55, fmt="(i8, a15, 3i8, f15.8)") node, "Syria", nint(Nodes(node)%HGPop), nint(Nodes(node)%FPalPop), nint(Nodes(node)%FNeoPop), fraction
    case (62)
        node = 7662
        fraction = 100. * Nodes(node)%FNeoPop / (Nodes(node)%FPalPop + Nodes(node)%FNeoPop)
        write(55, fmt="(i8, a15, 3i8, f15.8)") node, "Anatolia", nint(Nodes(node)%HGPop), nint(Nodes(node)%FPalPop), nint(Nodes(node)%FNeoPop), fraction
    case (79)
        node = 10347
        fraction = 100. * Nodes(node)%FNeoPop / (Nodes(node)%FPalPop + Nodes(node)%FNeoPop)
        write(55, fmt="(i8, a15, 3i8, f15.8)") node, "Hung-Croat", nint(Nodes(node)%HGPop), nint(Nodes(node)%FPalPop), nint(Nodes(node)%FNeoPop), fraction
    case (92)
        node = 10318
        fraction = 100. * Nodes(node)%FNeoPop / (Nodes(node)%FPalPop + Nodes(node)%FNeoPop)
        write(55, fmt="(i8, a15, 3i8, f15.8)") node, "SpainCard", nint(Nodes(node)%HGPop), nint(Nodes(node)%FPalPop), nint(Nodes(node)%FNeoPop), fraction
    case (95)
        node = 10662
        fraction = 100. * Nodes(node)%FNeoPop / (Nodes(node)%FPalPop + Nodes(node)%FNeoPop)
        write(55, fmt="(i8, a15, 3i8, f15.8)") node, "Portugal", nint(Nodes(node)%HGPop), nint(Nodes(node)%FPalPop), nint(Nodes(node)%FNeoPop), fraction
    case (97)
        node = 13040
        fraction = 100. * Nodes(node)%FNeoPop / (Nodes(node)%FPalPop + Nodes(node)%FNeoPop)
        write(55, fmt="(i8, a15, 3i8, f15.8)") node, "EastGerm", nint(Nodes(node)%HGPop), nint(Nodes(node)%FPalPop), nint(Nodes(node)%FNeoPop), fraction
        node = 13220
        fraction = 100. * Nodes(node)%FNeoPop / (Nodes(node)%FPalPop + Nodes(node)%FNeoPop)
        write(55, fmt="(i8, a15, 3i8, f15.8)") node, "WestGerm", nint(Nodes(node)%HGPop), nint(Nodes(node)%FPalPop), nint(Nodes(node)%FNeoPop), fraction
    case (102)
        node = 10854
        fraction = 100. * Nodes(node)%FNeoPop / (Nodes(node)%FPalPop + Nodes(node)%FNeoPop)
        write(55, fmt="(i8, a15, 3i8, f15.8)") node, "Navarre", nint(Nodes(node)%HGPop), nint(Nodes(node)%FPalPop), nint(Nodes(node)%FNeoPop), fraction
    case (169)
        node = 15387
        fraction = 100. * Nodes(node)%FNeoPop / (Nodes(node)%FPalPop + Nodes(node)%FNeoPop)
        write(55, fmt="(i8, a15, 3i8, f15.8)") node, "Sweden", nint(Nodes(node)%HGPop), nint(Nodes(node)%FPalPop), nint(Nodes(node)%FNeoPop), fraction
        close (55)
    end select
end subroutine SaveSites


end module Simulation