module StraightLine
    use GlobalVariables
    implicit none
    
    contains
    
    subroutine StraightLineSeaTravel(coastNode)
        !For a given coastNode, identifies all the possible sea travel
        !destinations following a straight line across the sea
        ! Uses: function DistNodes
        ! Uses: function SeaPath
        ! Uses: subroutine ResetTravelWeights
        !
        ! Used by: setTravelWeights
        implicit none
        integer :: coastNode
        integer, allocatable :: possibleDest(:)
        integer:: x,y, i, j, dest, numDest, maxDist
        real:: distance
        real :: nodeWeight
        integer:: oldNumDest
        
        maxDist = ceiling(maxSeaTravel)
        
        allocate(possibleDest(4*(maxDist+1)))
        x = Nodes(coastNode)%x
        y = Nodes(coastNode)%y
        
        numDest = 0
        
        do j = max(1,y-maxDist), min(y_numNodes, y+maxDist) !within y limits
            do i = max(1, x-maxDist), min(x_numNodes, x+maxDist) !witihn x limits
                if ((i/=x).or.(j/=y)) then
                    dest = nodesGrid(i,j)
                    if (Nodes(dest)%node_type == 3) then  !coast node -> possible destination
                        if (DistNodes(CoastNode, dest) > (maxSeaTravel + 0.01)) cycle !we are too far away
                        if (SeaPath(CoastNode, dest)) then
                            numDest = numDest + 1
                            possibleDest(numDest) = dest
                        endif
                    endif
                endif
            enddo
        enddo
        
        !All destinations are equally probable
        nodeWeight = (1-sum(Nodes(coastNode)%destinationWeight))/numDest

        !Record destination and weights        
        oldNumDest = Nodes(coastNode)%numDestinations
        Nodes(coastNode)%destinations(oldNumDest+1:oldNumDest + numDest) = possibleDest(:numDest)
        Nodes(coastNode)%destinationWeight(oldNumDest+1:oldNumDest + numDest) = nodeWeight
        Nodes(coastNode)%numDestinations = numDest+oldNumDest
        
        if (numDest == 0) then
            call ResetTravelWeights(coastNode)
        endif
        
    end subroutine StraightLineSeaTravel
    
    function DistNodes(origin, destination) result(distance)
    ! Computes the distance between two nodes in grid units.
        implicit none
        integer, intent(in) :: origin, destination
        real :: distance
        real :: deltaX, deltaY
        
        deltaX = real(Nodes(destination)%x - Nodes(origin)%x)
        deltaY = real(Nodes(destination)%y - Nodes(origin)%y)
        distance = sqrt(deltaX**2 + deltaY**2)
    end function DistNodes
    
    
    
    function SeaPath(origin, destination)
    ! Checks if there is a sea path from origin to destination
    ! Returns .true. or .false.
        implicit none
        integer, intent(in) :: origin, destination
        logical SeaPath
        integer :: deltaX, deltaY, i
        real :: x, y, stepX, stepY
        integer :: pathNode, nodePlus, nodeMinus
        real :: slope, islope  !slope= deltaY/deltaX, islope = deltaX/deltaY (inverse slope) 
        
        deltaX = Nodes(destination)%x - Nodes(origin)%x
        deltaY = Nodes(destination)%y - Nodes(origin)%y
        
        SeaPath = .true.
        
              
        if (deltaX /= 0) then
            x = real(Nodes(origin)%x)
            y = real(Nodes(origin)%y)
            slope = real(deltaY) / real(deltaX)
            stepX = abs(deltaX) / deltaX  !select
            
            do i = 1, abs(deltaX)-1
                x = x + stepX
                y = y + stepX * slope
                !check if its exactly a node, i.e., y is integer
                if (abs(y-nint(y)) < 0.000001) then
                    pathNode = nodesGrid(nint(x),nint(y))
                    if (Nodes(pathNode)%node_type /= 0) then
                        !If the node is not a sea node, there may not be a sea path.
                        !If it is a coast node, it might have a sea neighbor in the direction we are moving to.
                        !In that case there would be a path right along the coast.
                        nodePlus = nodesGrid(nint(x),nint(y+1))
                        nodeMinus = nodesGrid(nint(x),nint(y-1))
                        if ((Nodes(nodePlus)%node_type /= 0).and.(Nodes(nodeMinus)%node_type /= 0)) then
                            SeaPath = .false.  !None of the neighbors is sea, so there is no path
                            exit
                        endif
                    endif
                else
                    !Check the two nodes closer to the trajectory
                    nodePlus = nodesGrid(nint(x), ceiling(y))
                    nodeMinus = nodesGrid(nint(x), int(y))
                    if ((Nodes(nodePlus)%node_type /= 0).and.(Nodes(nodeMinus)%node_type /= 0)) then
                        SeaPath = .false.  !None of the neighbors is sea, so there is no path
                        exit
                    endif
                endif
            enddo
        
        endif
        
        !Only keep checking if there is still a seaPath
        if ((SeaPath == .true.).and.(deltaY /= 0)) then
            x = real(Nodes(origin)%x)
            y = real(Nodes(origin)%y)
            iSlope = real(deltaX) / real(deltaY)  !inverse slope
            stepY = abs(deltaY) / deltaY  !select
            
            do i = 1, abs(deltaY)-1
                y = y + stepY
                x = x + stepY * iSlope
                !check if its exactly a node, i.e., y is integer
                if (abs(x-nint(x)) < 0.000001) then
                    pathNode = nodesGrid(nint(x),nint(y))
                    if (Nodes(pathNode)%node_type /= 0) then
                        !If the node is not a sea node, there may not be a sea path.
                        !If it is a coast node, it might have a sea neighbor in the direction we are moving to.
                        !In that case there would be a path right along the coast.
                        nodePlus = nodesGrid(nint(x+1),nint(y))
                        nodeMinus = nodesGrid(nint(x-1),nint(y))
                        if ((Nodes(nodePlus)%node_type /= 0).and.(Nodes(nodeMinus)%node_type /= 0)) then
                            SeaPath = .false.  !None of the neighbors is sea, so there is no path
                            exit
                        endif
                    endif
                else
                    !Check the two nodes closer to the trajectory
                    nodePlus = nodesGrid(ceiling(x), nint(y))
                    nodeMinus = nodesGrid(int(x), nint(y))
                    if ((Nodes(nodePlus)%node_type /= 0).and.(Nodes(nodeMinus)%node_type /= 0)) then
                        SeaPath = .false.  !None of the neighbors is sea, so there is no path
                        exit
                    endif
                endif
            enddo

        endif
        
        !If we are following a path of slope +1 or -1 there might still be none existing paths considered available
        if ((SeaPath == .true.).and.(abs(deltaX) == abs(deltaY))) then  !Neighbors the path always crosses cells by their center
            x = real(Nodes(origin)%x)
            y = real(Nodes(origin)%y)
            stepX = abs(deltaX) / deltaX
            stepY = abs(deltaY) / deltaY
            do i = 1, abs(deltaX)
                x = x + stepX
                y = y + stepY
                pathNode = nodesGrid(nint(x),nint(y))
                if ((Nodes(pathNode)%node_type == 1).or.(Nodes(pathNode)%node_type == 2)) then !If inland, we cannot get there
                    seaPath = .false.
                    exit
                else if (Nodes(pathNode)%node_type == 3) then  !Check the diagonal nodes to see if we could get there
                    nodePlus = nodesGrid(nint(x),nint(y-stepY))  !stepY = +1 or -1
                    nodeMinus = nodesGrid(nint(x-stepX),nint(y)) !stepX = +1 or -1
                    !If none is a sea node, we cannot get there
                    if ((Nodes(nodePlus)%node_type /= 0).and.(Nodes(nodeMinus)%node_type /= 0)) then
                        SeaPath = .false.  !None of the neighbors is sea, so there is no path
                        exit
                    endif
                endif
            enddo
        endif
    end function SeaPath
    
    subroutine ResetTravelWeights(coastNode)
    !Used when, for some reason, no coastal neighbors were found
    implicit none
    integer(4), intent(in) :: coastNode
    integer(4) :: numDest
    real(8) :: expectedWeight, actualWeight, ratio
    
    numDest = Nodes(coastNode)%numDestinations
    if (numDest == 0) then
        Nodes(coastNode)%destinationWeight(0) = 1  !If there are no destinations (isolated islands) all the population stays
    else
        expectedWeight = 1-persistence
        actualWeight = sum(Nodes(coastNode)%destinationWeight(1:numDest))
        ratio = expectedWeight/actualWeight
        Nodes(coastNode)%destinationWeight(1:numDest) = ratio*Nodes(coastNode)%destinationWeight(1:numDest)
    endif
    
    
end subroutine ResetTravelWeights

    
end module StraightLine